/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.ApiFactory
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.talk.RxTalkApiClient
import com.kakao.sdk.template.DefaultTemplate
import com.kakao.sdk.network.RxApiInterceptor
import com.kakao.sdk.partner.model.*
import com.kakao.sdk.talk.model.Friends
import com.kakao.sdk.talk.model.Order
import io.reactivex.Single

/**
 * @author kevin.kang. Created on 17/04/2019..
 */

/**
 * @suppress
 */
val RxTalkApiClient.partnerApi: RxPartnerApi
    get() = ApiFactory.kapiWithOAuth.create(RxPartnerApi::class.java)

/**
 *
 */
fun RxTalkApiClient.friendsForPartner(
    friendType: FriendType? = null,
    friendFilter: FriendFilter? = null,
    friendOrder: FriendOrder? = null,
    secureResource: Boolean? = true,
    offset: Int? = null,
    limit: Int? = null,
    order: Order? = null
): Single<Friends<PartnerFriend>> =
    partnerApi.friends(friendType, friendFilter, friendOrder, offset, limit, order, secureResource)
        .compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())

/**
 * 카카오톡 채팅방 목록을 가져옵니다.
 */
fun RxTalkApiClient.chatList(
    filters: List<ChatFilter>? = null,
    secureResource: Boolean? = true,
    offset: Int? = null,
    limit: Int? = null,
    order: Order? = null
): Single<Chats> = partnerApi.chatList(
    filters?.joinToString(",") { it.value },
    offset, limit, order,
    secureResource
)
    .compose(RxApiInterceptor.handleApiError())
    .compose(apiInterceptor.handleApiError())

/**
 * 사용자의 카카오톡 채팅방에 속한 멤버를 조회합니다. 채팅방 아이디를 기준으로 조회하며 친구인 멤버만 조회할지 여부를 선택할 수 있습니다.
 */
fun RxTalkApiClient.chatMembers(
    chatId: Long,
    friendsOnly: Boolean? = null,
    includeProfile: Boolean? = null,
    secureResource: Boolean? = true,
    offset: Int? = null,
    limit: Int? = null,
    order: Order? = null
): Single<ChatMembers> = partnerApi.chatMembers(
    chatId,
    friendsOnly,
    includeProfile,
    offset,
    limit,
    order,
    secureResource
).compose(RxApiInterceptor.handleApiError())
    .compose(apiInterceptor.handleApiError())

/**
 * 개발자사이트에서 생성한 메시지 템플릿을 사용하여, 조회한 친구를 대상으로 카카오톡으로 메시지를 전송합니다.
 *
 * 템플릿을 생성하는 방법은 https://developers.kakao.com/docs/template 을 참고하시기 바랍니다.
 */
fun RxTalkApiClient.sendCustomMessageForPartner(
    receiverUuids: List<String>,
    templateId: Long,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), templateId, templateArgs)
        .compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())

/**
 * 개발자사이트에서 생성한 메시지 템플릿을 사용하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 *
 * 템플릿을 생성하는 방법은 https://developers.kakao.com/docs/template 을 참고하시기 바랍니다.
 */
fun RxTalkApiClient.sendCustomMessageToChats(
    receiverChatIds: List<Long>,
    templateId: Long,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> = partnerApi.sendMessage(
    "chat_id",
    KakaoJson.toJson(receiverChatIds),
    templateId,
    templateArgs
).compose(RxApiInterceptor.handleApiError())
    .compose(apiInterceptor.handleApiError())

/**
 * 기본 템플릿을 사용하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 */
fun RxTalkApiClient.sendDefaultMessageForPartner(
    receiverUuids: List<String>,
    templateParams: DefaultTemplate
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), templateParams)
        .compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())

/**
 * 기본 템플릿을 사용하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 */
fun RxTalkApiClient.sendDefaultMessageToChats(
    receiverChatIds: List<Long>,
    templateParams: DefaultTemplate
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage("chat_id", KakaoJson.toJson(receiverChatIds), templateParams)
        .compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())

/**
 * 지정된 URL을 스크랩하여, 조회한 친구를 대상으로 카카오톡으로 메시지를 전송합니다.
 *
 * 스크랩 커스텀 템플릿 가이드를 참고하여 템플릿을 직접 만들고 스크랩 메시지 전송에 이용할 수도 있습니다.
 *
 * @see RxTalkApiClient.friendsForPartner
 */
fun RxTalkApiClient.sendScrapMessageForPartner(
    receiverUuids: List<String>,
    requestUrl: String,
    templateId: Long? = null,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage(
        "uuid",
        KakaoJson.toJson(receiverUuids),
        requestUrl,
        templateId,
        templateArgs
    ).compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())

/**
 * 지정된 URL 을 스크랩하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 *
 * 스크랩 커스텀 템플릿 가이드를 참고하여 템플릿을 직접 만들고 스크랩 메시지 전송에 이용할 수도 있습니다.
 */
fun RxTalkApiClient.sendScrapMessageToChats(
    receiverChatIds: List<Long>,
    requestUrl: String,
    templateId: Long? = null,
    templateArgs: Map<String, String>? = null
): Single<PartnerMessageSendResult> =
    partnerApi.sendMessage(
        "chat_id",
        KakaoJson.toJson(receiverChatIds.map { it.toString() }),
        requestUrl,
        templateId,
        templateArgs
    ).compose(RxApiInterceptor.handleApiError())
        .compose(apiInterceptor.handleApiError())


