/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner

import com.google.gson.JsonObject
import com.kakao.sdk.common.ApiFactory
import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.util.Utility
import com.kakao.sdk.network.withClient
import com.kakao.sdk.partner.model.PartnerMessageSendResult
import com.kakao.sdk.talk.Constants
import com.kakao.sdk.partner.Constants as PartnerConstants
import io.reactivex.observers.TestObserver
import okhttp3.OkHttpClient
import okhttp3.mockwebserver.MockResponse
import okhttp3.mockwebserver.MockWebServer
import org.junit.jupiter.api.Assertions
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.params.ParameterizedTest
import org.junit.jupiter.params.provider.Arguments
import org.junit.jupiter.params.provider.MethodSource
import java.net.URLDecoder
import java.util.stream.Stream

/**
 * @author kevin.kang. Created on 17/04/2019..
 */
class RxPartnerApiTest {
    private lateinit var server: MockWebServer
    private lateinit var api: RxPartnerApi

    @BeforeEach
    fun setup() {
        server = MockWebServer()
        server.start()
        api = ApiFactory.withClient(server.url("/").toString(), OkHttpClient.Builder())
            .create(RxPartnerApi::class.java)
        val response = MockResponse().setResponseCode(200)
        server.enqueue(response)
    }

//    @CsvFileSource(resources = ["/csv/chat_list.csv"], numLinesToSkip = 1)
//    @ParameterizedTest
//    fun chatList(
//        fromId: Int? = null,
//        limit: Int? = null,
//        order: String? = null,
//        filter: String? = null
//    ) {
//        api.chatList(fromId, limit, order, filter)
//            .subscribe(TestObserver<Chats>())
//        val request = server.takeRequest()
//        val params = Utility.parseQuery(request.requestUrl?.query)
//
//        Assertions.assertEquals(fromId, params[Constants.FROM_ID]?.toInt())
//        Assertions.assertEquals(limit, params[Constants.LIMIT]?.toInt())
//        Assertions.assertEquals(order, params[Constants.ORDER])
//        Assertions.assertEquals(filter, params[Constants.FILTER])
//    }

    @MethodSource("sendMessageProvider")
    @ParameterizedTest
    fun sendMessage(
        receiverIdType: String,
        receiverIds: List<String>,
        templateId: Long,
        templateArgs: Map<String, String>?
    ) {
        api.sendMessage(
            receiverIdType,
            KakaoJson.toJson(receiverIds),
            templateId,
            templateArgs
        )
            .subscribe(TestObserver<PartnerMessageSendResult>())
        val request = server.takeRequest()
        val params = Utility.parseQuery(request.body.readUtf8())

        Assertions.assertEquals(receiverIdType, params[PartnerConstants.RECEIVER_ID_TYPE])
        Assertions.assertEquals(
            templateId.toString(),
            params[Constants.TEMPLATE_ID]
        )

        if (templateArgs == null) {
            Assertions.assertFalse(params.containsKey(Constants.TEMPLATE_ARGS))
            return
        }

        val decoded =
            URLDecoder.decode(params[Constants.TEMPLATE_ARGS], "UTF-8")
        val decodedJson = KakaoJson.fromJson<JsonObject>(decoded, JsonObject::class.java)
        Assertions.assertEquals(templateArgs.size, decodedJson.size())

        for ((k, v) in templateArgs) {
            Assertions.assertEquals(v, decodedJson[k].asString)
        }
    }

    companion object {
        @Suppress("unused")
        @JvmStatic
        fun sendMessageProvider(): Stream<Arguments> {
            return Stream.of(
                Arguments.of("user_id", listOf("1234"), 1234L, null),
                Arguments.of("uuid", listOf("1234"), 1234L, mapOf<String, String>()),
                Arguments.of("chat_id", listOf("1234"), 1234L, mapOf(Pair("key1", "value1")))
            )
        }
    }
}